@@ -3,6 +3,7 @@ import re
 import boto3
 import docker
 import yaml
+import json
 
 from sys import argv
 from textwrap import indent
@@ -12,24 +13,25 @@ from .utils import rgetattr, rsetattr
 
 
 class Deployment:
-    def __init__(self, path=""):
-        # Initialize paths and log settings
+    def __init__(self, path="", config=""):
         self.path = path
 
-        # Load the clients and config
-        with open(normpath(path)) as config:
-            self.config = yaml.safe_load(config)
-            self.clients = {
-                "os": os,
-                "s3": boto3.client("s3"),
-                "ecr": boto3.client("ecr"),
-                "sts": boto3.client("sts"),
-                "iam": boto3.client("iam"),
-                "events": boto3.client("events"),
-                "lambda": boto3.client("lambda"),
-                "session": boto3.session.Session(),
-                "docker": docker.client.from_env(),
-            }
+        self.config = yaml.safe_load(config)
+        if not self.config:
+            with open(normpath(path)) as config:
+                self.config = yaml.safe_load(config)
+
+        self.clients = {
+            "os": os,
+            "s3": boto3.client("s3"),
+            "ecr": boto3.client("ecr"),
+            "sts": boto3.client("sts"),
+            "iam": boto3.client("iam"),
+            "events": boto3.client("events"),
+            "lambda": boto3.client("lambda"),
+            "session": boto3.session.Session(),
+            "docker": docker.client.from_env(),
+        }
 
     def get_fully_qualified_node_names(self):
         return [f"{resource_name}.{action_name}"
@@ -39,6 +41,9 @@ class Deployment:
     def get_topologically_sorted_nodes(self, seen=set(), sorted_nodes=[]):
         for node_name in self.get_fully_qualified_node_names():
             def visit_parents(node_name):
+                if node_name in seen:
+                    return
+
                 node = rgetattr(self.config, node_name)
 
                 node.setdefault("name", node_name)
@@ -52,8 +57,7 @@ class Deployment:
 
                 sorted_nodes.append(node)
 
-            if node_name not in seen:
-                visit_parents(node_name)
+            visit_parents(node_name)
 
         return sorted_nodes
 
@@ -90,14 +94,14 @@ class Deployment:
 
             resource_name, action_name = node_name.split(".")
 
-            print(node_name)
+            print(f"\033[36m{node_name}\033[0m")
 
             try:
                 action = rgetattr(node_client, action_name)
                 value = action(**self.shared_lookup(node_args))
 
-                print(
-                    indent(f"setting value {value}", '\t+ ', lambda _: True))
+                print(indent(json.dumps(value, indent=2,
+                      default=str), '+ ', lambda _: True))
 
                 rsetattr(
                     context=self.config,
@@ -105,8 +109,8 @@ class Deployment:
                     value=value,
                 )
             except Exception as e:
-                print(
-                    indent(f"err: {e}", '\t- ', lambda _: True))
+                print(indent(json.dumps(e, indent=2, default=str),
+                      '- ', lambda _: True))
 
 
 def main():
